<?php
namespace Ak\Db\Adapter\Driver;

use Doctrine\DBAL\Driver;
use Doctrine\DBAL\Driver\PDOStatement;
use Doctrine\DBAL\DriverManager;
use Interop\Container\ContainerInterface;
use Laminas\Db\Adapter\Driver\Pdo\Statement;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;

/**
 * Stvara novu instancu traženog DB drivera.
 *
 * @package Ak\Db\Adapter\Driver
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
class DriverAbstractFactory implements AbstractFactoryInterface
{
    /**
     * @param string $requestedName
     * @return string
     */
    private function getDriverName(string $requestedName): string
    {
        return strtolower(preg_replace('/^db.driver./', '', $requestedName));
    }

    /**
     * Da li ova tvornica može stvoriti traženi driver?
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     *
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        $driverName = $this->getDriverName($requestedName);
        return in_array($driverName, ['mysql', 'sqlsrv', 'sqlite']);
    }

    /**
     * Stvara instancu.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return DriverInterface
     * @throws \Doctrine\DBAL\DBALException
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $options = $options ?? [];
        $driverName = $this->getDriverName($requestedName);

        if (!$this->canCreate($container, $requestedName)) {
            throw new \InvalidArgumentException(sprintf("Ne znam napraviti driver '%s'.", $driverName));
        }

        $driverClass = null;

        switch ($driverName) {
            case 'mysql':
                $driverClass = Dbal\DbalMysql::class;
                break;
            case 'sqlite':
                $driverClass = Dbal\DbalSqlite::class;
                break;
            case 'sqlsrv':
                $driverClass = Dbal\DbalSqlsrv::class;
                break;
        }

        /** @var Dbal\Connection $connection */
        $connection = DriverManager::getConnection([
            'driver' => 'pdo_' . $driverName,
            'user' => $options['username'] ?? '',
            'password' => $options['password'] ?? '',
            'host' => $options['hostname'] ?? 'localhost',
            'port' => $options['port'] ?? null,
            'dbname' => $options['database'] ?? '',
            'wrapperClass' => Dbal\Connection::class,
            'driverClass' => $driverClass,
        ]);

        $driver = $connection->getDriver();

        if ($driver instanceof DriverInterface) {
            $driver->setConnection($connection);
        }

        return $driver;
    }
}
