<?php
namespace AkTest\Db\TableGateway;

use Ak\Db\ConfigProvider;
use Ak\Db\TableGateway\TableGateway;
use Ak\Db\TableGateway\TableGatewayAbstractFactory;
use Ak\Test\AbstractTestCase;
use Ak\Test\DbTestTrait;
use AkTest\Db\TestConfigProvider;
use Doctrine\DBAL\Schema\Schema;
use PHPUnit\DbUnit\DataSet\ArrayDataSet;
use Laminas\Db\Adapter\AdapterInterface;

/**
 * Class TableGatewayAbstractFactoryTest
 *
 * @package AkTest\Db\TableGateway
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
class TableGatewayAbstractFactoryTest extends AbstractTestCase
{
    use DbTestTrait;

    public $configProviders = [
        ConfigProvider::class,
        TestConfigProvider::class,
    ];

    public function testCanCreateWillReturnFalseWhenCannotCreateTable()
    {
        $container = $this->getContainer();
        $factory = new TableGatewayAbstractFactory();

        $this->assertFalse($factory->canCreate($container, 'any_table'));
    }

    public function testCanCreateWillReturnTrueIfItCanCreateTable()
    {
        $container = $this->getContainer([
            'db' => [
                'tables' => [
                    'table.some' => [],
                ],
            ],
        ]);

        $factory = new TableGatewayAbstractFactory();

        $this->assertTrue($factory->canCreate($container, 'table.some'));
    }

    /**
     * @expectedException \RuntimeException
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testWillThrowExceptionWhenNoNamePresent()
    {
        $container = $this->getContainer([
            'db' => [
                'tables' => [
                    'table.some' => [],
                ],
            ],
        ]);

        $factory = new TableGatewayAbstractFactory();
        $factory($container, 'table.some');
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testWillInjectTheCorrectAdapter()
    {
        $adapter = $this->createMock(AdapterInterface::class);

        $container = $this->getContainer([
            'db' => [
                'tables' => [
                    'table.some' => [
                        'name' => 'some_table',
                    ],
                ],
            ],

            'dependencies' => [
                'services' => [
                    'db' => $adapter,
                ],
            ],
        ]);

        $factory = new TableGatewayAbstractFactory();
        /** @var TableGateway $table */
        $table = $factory($container, 'table.some');

        $this->assertSame($adapter, $table->getAdapter());
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testCanInjectSomeOtherAdapter()
    {
        $adapter = $this->createMock(AdapterInterface::class);
        $otherAdapter = $this->createMock(AdapterInterface::class);

        $container = $this->getContainer([
            'db' => [
                'tables' => [
                    'table.some' => [
                        'name' => 'some_table',
                        'adapter' => 'db.other',
                    ],
                ],
            ],

            'dependencies' => [
                'services' => [
                    'db' => $adapter,
                    'db.other' => $otherAdapter,
                ],
            ],
        ]);

        $factory = new TableGatewayAbstractFactory();
        /** @var TableGateway $table */
        $table = $factory($container, 'table.some');

        $this->assertSame($otherAdapter, $table->getAdapter());
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testCanCreateGatewayOfCustomClass()
    {
        $adapter = $this->createMock(AdapterInterface::class);
        $container = $this->getContainer([
            'db' => [
                'tables' => [
                    'table.some' => [
                        'name' => 'some_table',
                        'gateway' => MockTableGateway::class,
                    ],
                ],
            ],

            'dependencies' => [
                'services' => [
                    'db' => $adapter,
                ],
            ],
        ]);

        $factory = new TableGatewayAbstractFactory();
        /** @var TableGateway $table */
        $table = $factory($container, 'table.some');

        $this->assertInstanceOf(MockTableGateway::class, $table);
    }

    /**
     * @expectedException \RuntimeException
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testWillThrowExceptionForInvalidGatewayClass()
    {
        $adapter = $this->createMock(AdapterInterface::class);
        $container = $this->getContainer([
            'db' => [
                'tables' => [
                    'table.some' => [
                        'name' => 'some_table',
                        'gateway' => 'invalid',
                    ],
                ],
            ],

            'dependencies' => [
                'services' => [
                    'db' => $adapter,
                ],
            ],
        ]);

        $factory = new TableGatewayAbstractFactory();
        /** @var TableGateway $table */
        $factory($container, 'table.some');
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testWillAutoCreateTablesIfTheyDontExist()
    {
        $container = $this->getContainer([
            'db' => [
                'tables' => [
                    'db.table.example' => [
                        'name' => 'example_table',
                        'gateway' => MockTableGateway::class,
                    ],
                ],
            ],
        ]);

        $factory = new TableGatewayAbstractFactory();
        $factory($container, 'db.table.example');

        $conn = $this->getDbalConnection();
        $schema = $conn->getSchemaManager();

        $this->assertContains('example_table', $schema->listTableNames());
    }
}
